	program Surge_UV
      implicit real*8 (a-h,o-z)
C     SOLUTION OF THE 2DH TIDAL-WIND GENERATED CIRCULATION IN A COASTAL      
C     BASIN WITH COASTAL AND/OR OPEN SEA BOUNDARY CONDITIONS

C	More information about the code can be found in Vries et al. 1995, (Environ Softw), 
C	Krestenitis et al, 2011 (J. Coastal Management), Androulidakis et al 2015 (Dynamics of Oceans and Atmospheres)

      DIMENSION H(500,500),U(500,500),UN(500,500),V(500,500),cs(500,500)
     & ,VN(500,500),Z(500,500),WINX(800),WINY(800),ED(500,500)
     &,lontherm(500),
     & CB(500,500),VOR(500,500),URES(500,500),VRES(500,500),w(500,500)
      DIMENSION SXX(500,500),SXY(500,500),SYY(500,500),mask(500,500)
        integer n,day,t,tw,mon,twind,thour,iy,NN,dayf,ttt,startday,NU
        real*4 pp,hww,startime
           character*2 hh,mm,hhh
           character*3 ddd,yy,dddd
      character*8 ddate


	

       dimension alat(500,500),alon(500,500),radlat(500,500),cf(500,500)
       dimension dx(500,500),dy(500,500),wx(500,500),wy(500,500)
     &,zt(500,500),zz(500,500)
       dimension tsx(500,500),tsy(500,500),slp(500,500),slp1(500,500)
       dimension tsxx(500,500),tsyy(500,500),aalon(500),lonaeg(500)
       dimension sumz(500,500),zmeso(500,500),icoast(5000),jcoast(5000)
       DIMENSION Hoo(100),F(500),B(500),zb(500),zbb(50),ubb(50),vbb(50)
       character*5 TIDE
       CHARACTER*20 FNAME
       CHARACTER*2000 path

	write(*,*) 'Simulation started' 

	!set im, jm number of cells
	im=62
	jm=46
	

	!Set pi number
	pi=3.1415926535897932

   	        
	!Set simulation variables 

	 dt=1.		!DT=TIME	STEP	OF	THE	NUMERICAL	INTEGRATION
	 NM=86400.	!NM=NUMBER	OF	TIME	STEPS (NM=days*86400/dt - set the end of simulation)
	 zo=0.01	!ZO=SEA	BED	ROUGHNESS
	 ccc=0.1	!CONSTANT	OF	THE	HORIZONTAL	EDDY	VISCOSITY	COEFF.	
	 edd=5.		!EDD=MIN	VALUE	OF	THE	EDDY	VISCOSITY	COEFFICIENT
	 zino=0
	 zinu=0
	 zinl=0
	 zinr=0		!AMPLITUDES	OF	THE	TIDAL	COMPONENTS	ALONG	THE	LOWER	UPPER	LEFT	AND	RIGHT	OPEN	SEA	BOUNDARIES	(IF	ANY)
	 pero=10.
	 peru=10.
	 perl=10.
	 perr=10.	!PERIODS	OF	THE	CORRESPONDING	TIDAL	SIGNALS

C     **REMARK: EVEN IN CASE OF NO TIDAL INPUTS THE PERO,PERU,PERL,PERR PERIODS SHOULD
C     HAVE NON ZERO VALUES**


C     OPENING OF THE DEPTH DATA FILE AND INPUT OF THE DEPTH H(I,J) VALUES
C     REFERING TO THE CENTER OF EACH NODE

      open(60,file='Bathymetry.dat')
C	Bathymetry file must include the following format, where alon, alat, dx, dy and h are the longitude,latitude, x-grid size(m), y-grid size(m) and depth(m), respectively.  
	do jj=1,jm
	do ii=1,im
      read(60,*) i,j,alon(i,j),alat(i,j),dx(i,j),dy(i,j),h(i,j) 

	enddo
	enddo

C	*******Compute Coriolis force	

	do j=1,jm-1
	do i=1,im-1	
	radlat(i,j)=alat(i,j)*pi/180.
	wom=2.*pi/(24.*3600.)
	cf(i,j)=2.*wom*sin(radlat(i,j))

	enddo
	enddo

	!Set format of the timeseries output - first line name of station (e.g. Thessaloniki)

3000	format('Thessaloniki')
3001	format('Thessaloniki_U',x,'Thessaloniki_V')

C	*****************************
c     COMPUTATION OF THE BED FRICTION COEFFICIENTS CB(I,J) ACCORDING TO
C     THE ZO AND LOCAL DEPTH VALUES
      DO 555 J=1,JM-1
      DO 555 I=1,IM-1
      IF(H(I,J).GT.1) GOTO 556 
      CB(I,J)=0.
      GOTO 555
  556 CB(I,J)=(.4/(DLOG(H(I,J)/ZO)-1.))**2
c      if(H(i,j).gt.250) H(i,j)=250
  555 CONTINUE
c	********************************* 
    
C     ALOCATION OF THE MIN EDDY VALUES TO THE EDDY VISCOSITY MATRIX
      DO 445 J=1,JM-1
      DO 445 I=1,IM-1
  445 ED(I,J)=EDD3

C     COUNTER OF TIME STEPS
      do i=1,im  
      do j=1,jm  
      u(i,j)=0.0;v(i,j)=0.0;z(i,j)=0.0  
      enddo  
      enddo  

	
	! Open output files at stations
	 open(10,file='Stations_Elevations.dat')
       write(10,3000) 
	 open(11,file='Stations_Currents.dat')
       write(11,3001) 
	
	open(20,file='pressure.dat')
      open(21,file='Uwind.dat')
	open(22,file='Vwind.dat')


      do jj=1,jm
      do ii=1,im
      read(20,*)slp(ii,jj)
      read(21,*)wx(ii,jj)
      read(22,*)wy(ii,jj)

      end do
      end do
      close(20)
      close(21)
      close(22)

	N=0

	T=0
****************************Start Numerical  Integration*********************
  100 N=N+1
      T=T+DT
	

******************************
	dayz=N*dt/86400
	hww=float(t)/3600
      
       
        hw=T-3600*int(t/3600)
        if(hww.eq.int(hww)) then
        hhh=char(int(hww/10)+48)//char(int(hww-10*int(hww/10)+48))
	  endif
********open and read atmoshperic forcing ,winds and pressure
	


**************************************************************	

c******Calcualtion of Sea surface Drag Coefficiant by Smith&Banke,1975***

	do i=1,im-1
	do j=1,jm-1
	w(i,j)=SQRT(wx(i,j)**2+wy(i,j)**2)
	cs(i,j)=1.2/1025*(0.63+0.066*w(i,j))/10**3  ! sea surface drag coefficient
	
	enddo
	enddo

	! Computation of wind stresses

	do i=1,im-1
	do j=1,jm-1

	WY(i,j)=-2
	WX(i,j)=0

      TSX(i,j)=CS(i,j)*WX(i,j)*SQRT(WX(i,j)**2+WY(i,j)**2)
      TSY(i,j)=CS(i,j)*WY(i,j)*SQRT(WX(i,j)**2+WY(i,j)**2)
	enddo
	enddo


C	*********************************************
C     SOLUTION OF THE CONTINUITY EQUATION FOR THE COMPUTATION OF THE NEW
C     Z(I,J) VALUES IN THE MESH CENTERS
      DO 30 J=2,JM-2
      DO 30 I=2,IM-2

      IF(H(I,J).GT.1) GOTO 460
      Z(I,J)=0.
      GOTO 30
  460 CONTINUE
C     BLOCK FOR THE OPEN SEA BOUNDARY CONDITIONS. 
c	IF THE TIDAL COMPONENT IS NONZERO ON THAT BOUNDARY THE INCOMING AND 
c	RADIATED SIGNALS ARE SUPERIMPOSED. 
c	IF THE TIDAL COMPONENT IS ZERO THE RADIATED SIGNAL ONLY IS ESTIMATED
      IF((J.EQ.2).AND.(H(I,1).GT.1)) GOTO 110
      IF((J.EQ.JM-2).AND.(H(I,JM-1).GT.1)) GOTO 120
      IF((I.EQ.2).AND.(H(1,J).GT.1)) GOTO 130
      IF((I.EQ.IM-2).AND.(H(IM-1,J).GT.1)) GOTO 140

*************************************************
	z(i,j) = z(i,j)-dt*(((h(i+1,j)+h(i,j))*u(i+1,j)
     &                    -(h(i,j)+h(i-1,j))*u(i,j))/dx(i,j)
     &                    +((h(i,j+1)+h(i,j))*v(i,j+1)
     &                    -(h(i,j)+h(i,j-1))*v(i,j))/dy(i,j))*.5



      IF(ABS(Z(I,J)).LT..0000001) Z(I,J)=0.

      GOTO 30
  110 IF(ZINU.LT..01) GOTO 111
      CC=SQRT(9.81*H(I,J))
      EL=PERU*CC
	
      Z(I,J)=Z(I,J)+ZINU*SIN(6.2832*DT/PERU)*COS(6.2832*N*DT/PERU)+
     & CC*DT/DX(I,J)*(Z(I,J+1)-Z(I,J)+ZINU*SIN(6.2832*
     & DX(i,j)/EL)*COS(6.2832*(N-1)*DT/PERU))
      GOTO 30


  111 Z (I,J)=-V(I,J+1)*SQRT(H(I,J)/9.81)+(1012*100
     & -SLP(I,J))/(9.81*1025)
C	write(*,*)n,i,j,z(i,j)
C	pause
      GOTO 30
  120 IF(ZINO.LT..01) GOTO 121
	
      CC=SQRT(9.81*H(I,J))
      EL=CC*PERO
	
      Z(I,J)=Z(I,J)+ZINO*SIN(6.2832*DT/PERO)*COS(6.2832*N*DT/PERO)+
     & CC*DT/DX(i,j)*(Z(I,J-1)-Z(I,J)+ZINO*SIN(6.2832*
     & DX(i,j)/EL)*COS(6.2832*(N-1)*DT/PERO))
      GOTO 30

  121 Z (I,J)=V(I,J)*SQRT(H(I,J)/9.81)+(1012*100
     & -SLP(I,J))/(9.81*1025)
	
      GOTO 30

  130 IF(ZINL.LT..01) GOTO 131
	CC=SQRT(9.81*H(I,J))
      EL=CC*PERL

  	Z(I,J)=Z(I,J)+ZINL*SIN(6.2832*DT/PERL)*COS(6.2832*N*DT/PERL)+
     & CC*DT/DX(i,j)*(Z(I,J-1)-Z(I,J)+ZINL*SIN(6.2832*
     & DX(i,j)/EL)*COS(6.2832*(N-1)*DT/PERL))
     
  131	Z (I,J)=-U(I,J)*SQRT(H(I,J)/9.81)+(1012*100
     & -SLP(I,J))/(9.81*1025)

	
	GOTO 30

  140 IF(ZINR.LT..01) GOTO 141 
      CC=SQRT(9.81*H(I,J))
      EL=CC*PERR
	
      Z(I,J)=Z(I,J)+ZINR*SIN(6.2832*DT/PERR)*COS(6.2832*N*DT/PERR)+
     & CC*DT/DX(i,j)*(Z(I-1,J)-Z(I,J)+ZINR*SIN(6.2832*
     & DX(i,j)/EL)*COS(6.2832*(N-1)*DT/PERR))
      GOTO 30

 141  Z(I,J)=U(I,J)*SQRT(H(I,J)/9.81)+(1012*100
     & -SLP(I,J))/(9.81*1025)


   30 CONTINUE


c	******************************************************
	
C     SOLUTION OF THE EQUILIBRIUM EQUATION ALONG X AXIS FOR THE COMPUTATION
C     OF THE NEW U(I,J) COMPONENT ALONG THE MESH SIDES 
      DO 150 J=2,JM-2
      DO 150 I=3,IM-2

      IF((H(I-1,J).LT.2).OR.(H(I,J).LT.2)) GOTO 160
      GOTO 170
  160 UN(I,J)=0.
      GOTO 150
  170 HM=(H(I,J)+H(I-1,J))/2.
	TSXX(I,J)=0.5*(TSX(I,J)+TSX(I-1,J))
	

      VV=(V(I,J)+V(I-1,J)+V(I,J+1)+V(I-1,J+1))/4.
	
	CBB=(CB(I,J)+CB(I-1,J))/2.

      FRICT=CBB*U(I,J)*SQRT(U(I,J)**2+VV**2)
      DD=(ED(I,J)+ED(I,J+1))/2.

      ADVX=U(I,J)*(U(I+1,J)-U(I-1,J))/2./DX(i,j)				  
      ADVY=VV*(U(I,J+1)-U(I,J-1))/2./DY(i,j)
	
	! in case of numerical error --> pause
	if((frict.gt.100000)) then
	write(*,*)'FRICT=',n, i,j,FRICT,z(i,j),u(i,j),v(i,j)
	pause
	endif
	
	UN(I,J)=U (I,J)-DT*(ADVX+ADVY+9.81*(Z(I,J)-Z(I-1,J))/DX(i,j)
     &-CF(i,j)*VV+(slp(i,j)-slp(i-1,j))/(DX(I,J)*1025)
     & +FRICT/HM-TSXX(i,j)/HM-DD *(U (I+1,J)+U (I-1,J)+U (I,J+1)+U (I,
     & J-1)-4.*U (I,J))/(DX(i,j)*DY(I,J)))

      IF(ABS(UN(I,J)).LT..000001) UN(I,J)=0.
  
  150 CONTINUE

C     OPEN SEA BOUNDARY CONDITIONS OF ZERO VELOCITY GRADIENTS
      DO 180 J=2,JM-2
      IF(H(1,J).LT.2) GOTO 190
      UN(2,J)=UN(3,J)
  190 IF(H(IM-1,J).LT.2) GOTO 180
      UN(IM-1,J)=UN(IM-2,J)
  180 CONTINUE
      DO 200 I=2,IM-1
      IF((H(I,1).LT.2).OR.(H(I-1,1).LT.0.1)) GOTO 210
	UN(I,1)=UN(I,2)
  210 IF((H(I,JM-1).LT.2).OR.(H(I-1,JM-1).LT.2)) GOTO 200
      UN(I,JM-1)=UN(I,JM-2)
  200 CONTINUE
	
C     SOLUTION OF THE EQUILIBRIUM EQUATION ALONG Y FOR THE COMPUTATION OF
C     V(I,J) COMPONENTS ALONG THE MESH SIDES
      DO 250 J=3,JM-2
      DO 250 I=2,IM-2
      IF((H(I,J).LT.2).OR.(H(I,J-1).LT.2)) GOTO 260
      GOTO 270
  260 VN(I,J)=0.
      GOTO 250
  270 HM=(H(I,J)+H(I,J-1))/2.
      TSYY(I,J)=0.5*(TSY(I,J)+TSX(I,J-1))
      UU=(U(I,J)+U(I+1,J)+U(I,J-1)+U(I+1,J-1))/4.
      CBB=(CB(I,J)+CB(I,J-1))/2.
      FRICT=CBB*V(I,J)*SQRT(V(I,J)**2+UU**2)
      DD=(ED(I,J)+ED(I+1,J))/2.
      ADVX=UU*(V(I+1,J)-V(I-1,J))/2./DX(i,j)
      ADVY=V(I,J)*(V(I,J+1)-V(I,J-1))/2./DY(i,j)

	
      VN(I,J)=V(I,J)-DT*(CF(i,j)*UU+9.81*(Z(I,J)-Z(I,J-1))/DY(i,j)
     &+FRICT/HM + (slp(i,j)-slp(i,j-1))/(DY(I,J)*1025)
     & -TSYY(i,j)/HM-DD *(V (I,J+1)+V (I,J-1)+V (I+1,J)+V (I-1,J)-4*
     & V (I,J))/(DX(i,j)*DY(I,J))+ADVX+ADVY)

      IF(ABS(VN(I,J)).LT..000001) VN(I,J)=0.
  250 CONTINUE

	
C     OPEN BOUNDARY CONDITIONS OF ZERO VELOCITY GRADIENTS
      DO 280 I=2,IM-2
      IF(H(I,1).LT.2) GOTO 290
      VN(I,2)=  VN(I,3)
  290 IF(H(I,JM-1).LT.2) GOTO 280
      VN(I,JM-1)=VN(I,JM-2)
  280 CONTINUE
      DO 305 J=2,JM-1
      IF((H(1,J).LT.2).OR.(H(1,J-1).LT.2)) GOTO 310
      VN(1,J)=VN(2,J)
  310 IF((H(IM-1,J).LT.2.).OR.(H(IM-1,J-1).LT.2.)) GOTO 305
      VN(IM-1,J)=VN(IM-2,J)
  305 CONTINUE
      DO 400 J=1,JM-1
      DO 400 I=1,IM-1
      V(I,J)=VN(I,J)
  400 U(I,J)=UN(I,J)
C     COMPUTATION OF THE EDDY VISCOSITY DISTRIBUTION ACCORDING TO 
C     SMAGORINSKY ED=CCC*DX**2*NORM(GRAD(VORTICITY)))
      DO 444 J=2,JM-2
      DO 444 I=2,IM-2
      VOR(I,J)=(U(I,J)-U(I,J-1))/dy(i,j-1)-(V(I,J)-V(I-1,J)/dx(i-1,j))
  444 CONTINUE

      DO 4451 J=2,JM-2
      DO 4451 I=2,IM-2
          
      ED(I,J)=SQRT((VOR(I+1,J)-VOR(I-1,J))**2+(VOR(I,J+1)-VOR(I,J-1))
     & **2)*DX(i,j)*DY(i,j)*CCC

		
      IF(ED(I,J).LT.EDD) ED(I,J)=EDD
 4451 CONTINUE
    



**************************Write hourly output****************
	 if(hw.eq.0) then      
	 


           write(10,57)z(26,42)
		 write(11,58)u(26,42),v(26,42)	


************************write horizontal values ***************

         open(983,file='Horizontal_fields_'//hhh//'.dat')
         write(*,*)'Horizontal_fields_'//hhh//'.dat'
      
        do j=1,jm
	  do i=1,im


        write(983,984)i,j,alon(i,j),alat(i,j),u(i,j),v(i,j),z(i,j)
	
	
	  enddo
        enddo

        close(983)
        endif

  984 format(2i5,5f9.4)
   57 format(f10.5)
   58 format(f10.5,x,f10.5)






c	*********************end*******************
C	*******************************************

101	IF(N/10000*10000.NE.N) GOTO 100

C     COMPUTATION OF THE KINETIC ENERGY
      EK=0
      DO 410 I=2,IM-2
      DO 410 J=2,JM-2
  410 EK=EK+H(I,J)*((V(I,J)+V(I,J+1))**2+(U(I,J)+U(I+1,J))**2)/8

c
	IF(N.LT.NM) GOTO 100



102	stop
	End

